/*
  Author:  Jonathan Bondzie, Dr. Chuck Yeung
           Lennard Gas Simulation Source

	   Copyright (C) 1999  Penn State University

	   This program is free software; you can redistribute it and/or
	   modify it under the terms of the GNU General Public License
	   as published by the Free Software Foundation;
	   
	   This program is distributed in the hope that it will be useful,
	   but WITHOUT ANY WARRANTY; without even the implied warranty of
	   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	   GNU General Public License for more details.
	   
	   You should have received a copy of the GNU General Public License
	   along with this program; if not, write to the Free Software
	   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
	   02111-1307, USA.

	   email jab341@psu.edu
*/

#include "motifR.h"
#include "Headers.h" 
#include "kiss.h"
  
#define SQUARE(x)  (x*x)
#define vconstant 299.3793626

typedef long unsigned int myshort;

Widget  dgr_nparticles;         /* widget for the number of particles */
Widget  dgr_lx;                 /* widget for the width */
Widget  dgr_ly;                 /* widget for the length */
Widget  dgr_time_step;          /* widget for the time step */
Widget  dgr_finish_time;
Widget  dgr_graphics_interval;  
Widget  dgr_pressure_interval;
Widget  dgr_seed1;              /* widget for the randomly generated seeds */
Widget  dgr_seed2;
Widget  dgr_seed3;
Widget  dgr_gravity;
Widget  dgr_blennard;
Widget  dgr_temp;     
Variables* fptr;
Widgets*   wptrs;


/* function prototype */

void CreateFormChildren(Widget* widget,        /* widget to create */
			Widget* label,         /* lable for a particular widget */
			int top,int bottom,    /* top, bottom attachment */   
			char* string,          /* contains the label string */
			float initial_val      /* initial value */
			);

void CreationFunction(Widget* dgr_form,         /* parent widget for the other widgets*/
		      Widget *ptr2widget,       /* holds the widget name for creation*/
		      Widget *ptr2label_widget, /* holds the label widget */
		      int top,                  /* top position */ 
		      int bottom,               /* bottom position */
		      char* string,             /* label for the widget */
		      float initial_val         /* the initial value */
		      );

void XmgrHistFunction(void);
void GraphPushLabel(Widget* widget,char* string);
void RestoreString(Widget *widget, int* parameter);
void RestoreString(Widget *widget, float* parameter);
void RestoreString(Widget *widget, myshort* parameter);
void OkCallback(Widget widget,XtPointer client_data,XtPointer call_data);
void RefreshCallback(Widget widget,XtPointer client_data,XtPointer call_data);
void DismissCallback(Widget widget,XtPointer client_data,XtPointer call_data);
void AssembleChildren(Widget* widget,Widget* label,int* parameter,char* string);
void AssembleChildren(Widget* widget,Widget* label,float *parameter,char* string);
void AssembleChildren(Widget* widget,Widget* label,myshort *parameter,char* string);
void FormPushButtonCallback(Widget widget,XtPointer client_data,XtPointer call_data);


/* these are variables needed by the form creation widget */

int top =-8,  
  bottom =0,
  left   =0,
  right  =0;
Cardinal n =0;  
Arg args[10]; 

/*************************************************************************************

  Form creation routines. Everthing that has to do with the form widget and 
  its children are here; except the form widget itself...
  

*/

/* initial system temperature calculation routine */

float InitialTemp(void)
{
  float tp =0;  

  for(int i=0; i<(*fptr).nparticles; i++ )
  {
    tp+= ( (*fptr).vx[i] * (*fptr).vx[i] + (*fptr).vy[i] * (*fptr).vy[i] )/2;
  }
  tp= (tp * 862.507157)/(*fptr).nparticles /*blennard*/;
  return(tp);
}

/* dismiss callback- unmanages the form widget */

void DismissCallback(Widget widget,XtPointer client_data,XtPointer call_data)
{
  widget = widget;
  client_data = client_data;
  call_data = call_data;
  XtUnmanageChild((*wptrs).dgr_form);
}


/* refresh callback*/

void RefreshCallback(Widget widget,XtPointer client_data,XtPointer call_data)
{
  widget = widget;
  client_data = client_data;
  call_data = call_data;
  SetTextString(dgr_lx,(double)(*fptr).lx);
  SetTextString(dgr_ly,(double)(*fptr).ly);
  SetTextString(dgr_graphics_interval,(double)(*fptr).update_interval);
  SetTextString(dgr_time_step,(double)(*fptr).dt);
  SetTextString(dgr_finish_time,(double)(*fptr).end_time);
  SetTextString(dgr_pressure_interval,(double)(*fptr).ftime_interval);
  SetTextString(dgr_temp,(*fptr).temp);
}

/* just a routine to set the text string */

void RestoreString(Widget *widget, float* parameter)
{
  float  number;
  char*   string = new char[32];
  float initial_temp =0;

  string = GetTextString(*widget);
  number =(float) atof(string);
  *parameter = number;
  
  if(widget == &dgr_finish_time)
    SetTextString((*wptrs).dgr_endtime_text,(*fptr).end_time);
 
  if(widget == &dgr_temp)
  { 
    initial_temp= InitialTemp();
    number = number / initial_temp;
  
    for(int i=0;i<(*fptr).nparticles;i++)
    {
      (*fptr).vx[i] = (*fptr).vx[i] * sqrt(number);
      (*fptr).vy[i] = (*fptr).vy[i] * sqrt(number);
    }
  }
  delete string;
}

void RestoreString(Widget *widget, int* parameter)
{
  int  number;
  char* string = new char[32];

  string = GetTextString(*widget);
  
  number =(int) atof(string);
  
  if((*fptr).nparticles != (int)number)
  {
    if(number / ( (*fptr).lx * (*fptr).ly ) > 0.4)
    {
      strcpy(string,"too many particles for this box size");
      SetTextString(dgr_nparticles, (*fptr).nparticles);
      MessageDialogCreationFunction((*wptrs).dgr_message_window,string);   
    }
    else
    {
      *parameter = number;
      (*fptr).t=0;

      /* read all the present parameters */
      OkCallback((*wptrs).dgr_form,(XtPointer)&dgr_nparticles,(XtPointer)NULL);
      InitializeArrays(fptr);
      make_initial_state(fptr);
      Neighbour_List(fptr); 
      (*fptr).reset = TRUE;
      (*fptr).temp =0;
    }
  }
  delete string;
}

void RestoreString(Widget *widget, myshort* parameter)
{
  myshort  number;
  char*   string = new char[32];
  
  string = GetTextString(*widget);
  number =(myshort) atof(string);
  *parameter = number ;
  delete string;
  
}


/* reads in all the values and then unmanages the form widget */
   
void OkCallback(Widget widget,XtPointer client_data,XtPointer call_data)
{
  widget = widget;
  call_data = call_data;
  client_data = client_data;
  RestoreString(&dgr_lx,&(*fptr).lx);
  RestoreString(&dgr_ly,&(*fptr).ly);
  RestoreString(&dgr_time_step,&(*fptr).dt);
  RestoreString(&dgr_finish_time,&(*fptr).end_time);
  RestoreString(&dgr_graphics_interval,&(*fptr).update_interval);
  RestoreString(&dgr_pressure_interval,&(*fptr).ftime_interval);
  RestoreString(&dgr_gravity,&(*fptr).g);
  RestoreString(&dgr_blennard,&(*fptr).blennard);
  RestoreString(&dgr_temp,&(*fptr).temp);
  RestoreString(&dgr_nparticles,&(*fptr).nparticles);
  RestoreString(&dgr_seed1,&(*fptr).seed1);
  RestoreString(&dgr_seed2,&(*fptr).seed2);
  RestoreString(&dgr_seed3,&(*fptr).seed3);
  
  /* if any of the system parameters get changed the whole 
     intergration starts over.                             */
  (*fptr).reset = TRUE;
}


/* it's almost sick that I should have to write so many functions
   to have this thing done. But I really am all for modularity so ... */

void AssembleChildren(Widget* widget,Widget* label,float *parameter,char* string)
{
  top = top +8;
  bottom = bottom +8;
  CreateFormChildren(widget,label,top,bottom,string,*parameter);
}

void AssembleChildren(Widget* widget,Widget* label,int* parameter,char* string)
{
 top = top +8;
 bottom = bottom +8;
 CreateFormChildren(widget,label,top,bottom,string,*parameter);
}

void AssembleChildren(Widget* widget,Widget* label,unsigned long* parameter,char* string)
{
 top = top +8;
 bottom = bottom +8;
 CreateFormChildren(widget,label,top,bottom,string,*parameter);
}


/* routine for creating the radio buttons */
void RadioButtonCallback(Widget* ptr2label,Widget* parent,Widget* fparent,Widget* child,char* string)
{ 
  static int ONCE = TRUE;
 
  if(ONCE == TRUE)
  {
    *ptr2label = XtVaCreateManagedWidget("Toggle",
					 xmLabelWidgetClass, /* creation widget to call */
					 (*wptrs).dgr_form,  /* attaches to this widget */
					 XmNtopAttachment,   /* this will attach the top of the label
								widget to the next specified position */
					 XmATTACH_FORM,      /* attach to the form */
					 XmNtopPosition,
					 top,
					 XmNbottomAttachment,
					 XmATTACH_POSITION,  /* attach to this position */
					 XmNbottomPosition,
					 bottom,
					 XmNleftAttachment,
					 XmATTACH_FORM,
					 XmNrightAttachment,
					 XmATTACH_POSITION,
					 XmNrightPosition,
					 50,
					 NULL                /* end the argument list */
					 ); 
    ONCE = FALSE;
  }

  /* create a radio button */
  
  *fparent = XtVaCreateManagedWidget("Radio",
				    xmRowColumnWidgetClass,
				    (*wptrs).dgr_form,
				    XmNtopAttachment,
				    XmATTACH_POSITION,
				    XmNtopPosition,
				    top,
				    XmNbottomAttachment,
				    XmATTACH_POSITION,
				    XmNbottomPosition,
				    bottom,
				    XmNleftAttachment,
				    XmATTACH_POSITION,
				    XmNleftPosition,
				    left,
				    XmNrightAttachment,
				    XmATTACH_POSITION,
				    XmNrightPosition,
				    right,
				    NULL); 

  /* create a radio button */

  *parent = XmCreateRadioBox(*fparent,"Radio",NULL,0);

  /* create the radio button */
  
  *child = XtVaCreateManagedWidget(string,
				   xmToggleButtonGadgetClass,
				   *parent,
				   NULL
				   );
  XtManageChild(*child);
  XtManageChild(*parent);
}

/* sets a switch to display the instant temperature and pressure */

Widget  dgr_instant;            /* instant toggle widget */
Widget  dgr_average;            /* averaget toggle widget */
void RadioButtonInstantCallback(Widget widget,XtPointer client_data,XtPointer call_data)
{
  
  call_data = call_data;
  widget = widget;
  client_data = client_data;
  (*fptr).display_integral = 0;
  XtVaSetValues(dgr_instant,XmNset,TRUE,NULL);
  XtVaSetValues(dgr_average,XmNset,FALSE,NULL);
}

/* sets a switch to display the integrated temperature and pressure */

void RadioButtonAverageCallback(Widget widget,XtPointer client_data,XtPointer call_data)
{
  client_data = client_data;
  call_data = call_data;
  widget = widget;
  (*fptr).display_integral = 1; 
  XtVaSetValues(dgr_average,XmNset,TRUE,NULL);
  XtVaSetValues(dgr_instant,XmNset,FALSE,NULL);
}


/* this function calls all the children that need be created */

void CallFormChildren(void)
{
  char* string = new char[32];
  Widget  dgr_Plabel;             /* particle label */
  Widget  dgr_lx_label;
  Widget  dgr_ly_label;
  Widget  dgr_tlabel;             /* time step label */
  Widget  dgr_flabel;             /* endtime label */
  Widget  dgr_glabel;             /* graphics interval label */
  Widget  dgr_plabel;             /* pressure label */
  Widget  dgr_seed1_label;        /* seed labels */
  Widget  dgr_Glabel;             /* gravity label */
  Widget  dgr_blabel;             /* blennard lablel */  
  Widget  dgr_Tlabel;
  Widget  dgr_refresh;            /* refresh form widget, refreshes the form widget */
  Widget  dgr_accept;             /* ok form widget, closes the form widget */
  Widget  dgr_dismiss;
  Widget  dgr_it_parent;          /* the parent widget for the instant toggle button */
  Widget  dgr_at_parent;          /* the parent widget for the average toggle button */
  Widget  dgr_pane1;              /* parent widgets of the toggles buttons */
  Widget  dgr_pane2;
  Widget  dgr_toggle_label;

  left = 50;
  right = 100;
  
  wptrs = ReturnClassAddress();
  strcpy(string,"nparticles");
  AssembleChildren(&dgr_nparticles,&dgr_Plabel,&(*fptr).nparticles,string);

  strcpy(string,"lx");
  AssembleChildren(&dgr_lx,&dgr_lx_label,&(*fptr).lx,string);
  
  strcpy(string,"ly");
  AssembleChildren(&dgr_ly,&dgr_ly_label,&(*fptr).ly,string);
  
  strcpy(string,"gravity");
  AssembleChildren(&dgr_gravity,&dgr_Glabel,&(*fptr).g,string);

  strcpy(string,"blennard");
  AssembleChildren(&dgr_blennard,&dgr_blabel,&(*fptr).blennard,string);

  (*fptr).temp = InitialTemp();
  strcpy(string,"sys temp");
  AssembleChildren(&dgr_temp,&dgr_Tlabel,&(*fptr).temp,string);
  
  strcpy(string,"time step");
  AssembleChildren(&dgr_time_step,&dgr_tlabel,&(*fptr).dt,string);

  strcpy(string,"endtime");
  AssembleChildren(&dgr_finish_time,&dgr_flabel,&(*fptr).end_time,string);

  strcpy(string,"graphics in'val");
  AssembleChildren(&dgr_graphics_interval,&dgr_glabel,&(*fptr).update_interval,string);

  strcpy(string,"pressure in'val");
  AssembleChildren(&dgr_pressure_interval,&dgr_plabel,&(*fptr).ftime_interval,string);
 
  right = 66;
  strcpy(string,"seeds");
  AssembleChildren(&dgr_seed1,&dgr_seed1_label,&(*fptr).seed1,string);

  left = right;
  right = 82;
  top = top -8;
  bottom = bottom -8;
  AssembleChildren(&dgr_seed2,&dgr_seed1_label,&(*fptr).seed2,string);
 
  left = right;
  right = 100;
  top = top -8;
  bottom = bottom -8;
  AssembleChildren(&dgr_seed3,&dgr_seed1_label,&(*fptr).seed3,string);

  left = 50;
  right = 75;
  top = top +6;
  bottom = bottom +6;
  strcpy(string,"instant");
  RadioButtonCallback(&dgr_toggle_label,&dgr_it_parent,&dgr_pane1,&dgr_instant,string);
  XtAddCallback(dgr_instant,XmNvalueChangedCallback,RadioButtonInstantCallback,NULL);

  left = right;
  right = 100;
  strcpy(string,"integ");
  RadioButtonCallback(&dgr_toggle_label,&dgr_at_parent,&dgr_pane2,&dgr_average,string);
  XtAddCallback(dgr_average,XmNvalueChangedCallback,RadioButtonAverageCallback,NULL);
 
  left = 0;
  right = 34;
  strcpy(string,"refresh");
  GraphPushLabel(&dgr_refresh,string);
  XtAddCallback(dgr_refresh,
		 XmNactivateCallback,
		 RefreshCallback,
		 NULL
		 );

  left = right;
  right = 67;
  strcpy(string,"accept");
  GraphPushLabel(&dgr_accept,string);
  XtAddCallback(dgr_accept,
		 XmNactivateCallback,
		 OkCallback,
		 NULL
		 );
  left = right;
  right = 100;
  strcpy(string,"dismiss");
  GraphPushLabel(&dgr_dismiss,string);
  XtAddCallback(dgr_dismiss,
		XmNactivateCallback,
		DismissCallback,
		NULL
		 );
  XtVaSetValues(dgr_average,XmNset,TRUE,NULL);
  delete string;
}


/* callback function for the formpushbutton widget ...
   it just manages the form widget                     */

void FormPushButtonCallback(Widget widget,
			    XtPointer client_data,
			    XtPointer call_data
			    )

{
  widget = widget;
  call_data = call_data;
  client_data = client_data;
  RefreshCallback((*wptrs).dgr_form,(XtPointer)NULL,(XtPointer)NULL);
  XtManageChild((*wptrs).dgr_form);
}
  

  
/* this is a routine that gets and sets text strings in 
   some given widget.                                   */

void SetTextString(Widget widget,double string)
{
  char* new_string = new char[32];  /* contains the converted string */
  int max_length;                   /* has the maximum number of charcters
				       stored in any one widget at some time */
  gcvt(string,(*fptr).ndigit,new_string);
  max_length=XmTextGetLastPosition(widget);
  XmTextReplace(widget,(XmTextPosition) 0,(XmTextPosition)
  max_length,new_string);
  delete new_string;
}
    


/* function that extracts a string stored in a widget */

char* GetTextString(Widget widget)
{
  char* old_string = new char[32];
  old_string = XmTextGetString(widget);
  return old_string;
}



 
/*********************************************************************************************
 *  form widget creation utils, not !                                           
 *  this part of the code contains all the widgets and gadgets                  
 *  there ever will be to create the initial conditions for the                 
 *  graphics routine. Zort !!!                                                  
 *********************************************************************************************/

void CreationFunction(Widget *dgr_form,         /* we need the form widget to create */
		      Widget *ptr2widget,       /* holds the widget name for creation*/
		      Widget *ptr2label_widget, /* holds the label widget */
		      int top,                  /* top position */ 
		      int bottom,               /* bottom position */
		      char* string,             /* label for the widget */
		      float initial_val
		      )
{
 
  dgr_form = dgr_form;
  if( (ptr2widget != &dgr_seed2) || (ptr2widget != &dgr_seed3))
  {
    *ptr2label_widget = XtVaCreateManagedWidget(string,
						xmLabelWidgetClass, /* creation widget to call */
						(*wptrs).dgr_form,           /* attaches to this widget */
						XmNtopAttachment,   /* this will attach the top of the label
								       widget to the next specified position */
						XmATTACH_POSITION,  /* attach to the form */
						XmNtopPosition,
						top,
						XmNbottomAttachment,
						XmATTACH_POSITION,  /* attach to this position */
						XmNbottomPosition,
						bottom,
						XmNleftAttachment,
						XmATTACH_FORM,
						XmNrightAttachment,
						XmATTACH_POSITION,
						XmNrightPosition,
						50,
						NULL                /* end the argument list */
						); 
  }
  
  *ptr2widget = XtVaCreateManagedWidget(string,
					xmTextWidgetClass, 
					(*wptrs).dgr_form,
					XmNeditMode,
					XmSINGLE_LINE_EDIT,
					XmNtopAttachment,
					XmATTACH_POSITION,
					XmNtopPosition,
					top,
					XmNbottomAttachment,
					XmATTACH_POSITION,
					XmNbottomPosition,
					bottom,
					XmNleftAttachment,
					XmATTACH_POSITION,
					XmNleftPosition,
					left,
					XmNrightAttachment,
					XmATTACH_POSITION,
					XmNrightPosition,
					right,
					NULL);
  SetTextString(*ptr2widget,initial_val); 
} /* end of the form creation function */


/***********************************************************************************
 * this routine creates the children of the form widget, PS it uses the routine    *
 * just above.                                                                     * 
 ***********************************************************************************/
void CreateFormChildren(Widget *widget,Widget *label,int top,int bottom,char* string,
			float initial_val)
{
  
  /* create one form widget, then pass the address space of the form
     widget to the subsequent widget creation functions */
  
  CreationFunction(&(*wptrs).dgr_form,
		   widget,
		   label,
		   top,
		   bottom,
		   string,
		   initial_val
		   );
} 

/* this function creates a label and a pushbutton for the
   Maxwell Boltzman distribution.                          */

void GraphPushLabel(Widget* widget,char* string)
{
  XmString label_string = XmStringCreateSimple(string);

  *widget = XtVaCreateManagedWidget(string,
				      xmPushButtonWidgetClass, 
				      (*wptrs).dgr_form,					
				      XmNtopAttachment,
				      XmATTACH_POSITION,
				      XmNtopPosition,
				      bottom,
				      XmNbottomAttachment,
				      XmATTACH_FORM,
				      XmNleftAttachment,
				      XmATTACH_POSITION,
				      XmNleftPosition,
				      left,
				      XmNrightAttachment,
				      XmATTACH_POSITION,
				      XmNrightPosition,
				      right,
				      XmNlabelString,
				      label_string,
				      NULL);
   
}


/* create a message dialog */

void MessageDialogCreationFunction(Widget parent,char* string)
{
  Widget message_dialog,child;
  XmString  new_string;

  new_string = XmStringCreateSimple(string);
  n=0;
  XtSetArg(args[n],XmNmessageString,new_string);n++;
  message_dialog = XmCreateInformationDialog(parent,"info",args,n);
  child = XmMessageBoxGetChild(message_dialog,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(child);
  child = XmMessageBoxGetChild(message_dialog,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(child);
  XtManageChild(message_dialog);
    
}


/*************************************************************************************
  
  
  these subroutines are for graphing or plotting and what nots.
  the Maxwell Boltzmand distribution stuff ... 

  */

float hist_sum =0;                       /* sum of all the histogram values */
static int ok = 0;

/* this routine updates the histogram based on some elapsed 
 udate interval. It alse creates a histogram file and callse 
 the xmgr program ...                                         */

void XmgrHistFunction(void)
{

  hist_sum = 0;

  switch(ok)
  {
     case 0:
       break;
     case 1:
     {
       for(int i=0;(*fptr).histogram[i] != 0;i++)
	 hist_sum += (*fptr).histogram[i];
       
       for(int u=0;(*fptr).histogram[u] !=0;u++)
       { 
	 /* write the histogram */
	 cout<<(u*(*fptr).binsize*vconstant)/*(sqrt(blennard))*/<<" "<<(*fptr).histogram[u]/hist_sum<<endl;       }
       cout<<"&"<<endl;
       cout<<"@title "<<"\""<<" Maxwell Bolzman distribution "<<"\""<<endl;
       cout<<"@subtitle "<<"\""<<" Probability of number particles at some velocity"<<"\""<<endl;
       cout<<"@ xaxis  label "<<"\""<<" Velocity (m/s)"<<"\""<<endl;
       cout<<"@ yaxis  label "<<"\""<<" Probability "<<"\""<<endl;
       cout<<"@ autoscale"<<endl;
       cout<<"@ kill s0"<<endl;
     }
  }
}/* emd of the routine */


/* this routine determins what to do, based on the mask */

void Function(Variables* ptr)
{

  /* the values of the mask is like this:-

     11:- there is a request and there is initially thought to be 
     enough data. this may however not be the case.

     10:- there is a request and there is not enough data.
     
     01:- there is no request but there is enough data, this is possible
     because the histogram is cumulative.

     00:- all requests have been satisfied
   */

  switch ((*ptr).mask)
  {
     case 11:
     {
       XmgrHistFunction();
       break;
     }
     case 01:
     {
       XmgrHistFunction();
       break;
     }
     case 00:
     {
       /*XmgrHistFunction();
       break;*/ 
     }
  }
  fptr = ptr;
  
  /* XmgrHistFunction() gets called before the address of fptr is set equal
     to ptr. This causes the system to crash,since it doesn't know what 
     the true values of histogram[] array are. this is to correct the
     situation.i.e. XmgrHistFunction() will not read or write to histogram[]
     untill fptr has the correct address of the Variables class.            */
  ok = 1;
  
}  




















